@@ -16,8 +16,10 @@ import torch
 import torch.nn as nn
 
 from pytorch_widedeep.wdtypes import *  # noqa: F403
-from pytorch_widedeep.models.tab_mlp import MLP, get_activation_fn
-from pytorch_widedeep.models.tabnet.tab_net import TabNetPredLayer
+from pytorch_widedeep.models.fds_layer import FDSLayer
+from pytorch_widedeep.models._get_activation_fn import get_activation_fn
+from pytorch_widedeep.models.tabular.mlp._layers import MLP
+from pytorch_widedeep.models.tabular.tabnet.tab_net import TabNetPredLayer
 
 warnings.filterwarnings("default", category=UserWarning)
 
@@ -27,8 +29,7 @@ device = torch.device("cuda" if use_cuda else "cpu")
 
 class WideDeep(nn.Module):
     r"""Main collector class that combines all ``wide``, ``deeptabular``
-    (which can be a number of architectures), ``deeptext`` and
-    ``deepimage`` models.
+    ``deeptext`` and ``deepimage`` models.
 
     There are two options to combine these models that correspond to the
     two main architectures that ``pytorch-widedeep`` can build.
@@ -42,87 +43,83 @@ class WideDeep(nn.Module):
     Parameters
     ----------
     wide: ``nn.Module``, Optional, default = None
-        ``Wide`` model. I recommend using the ``Wide`` class in this
-        package. However, it is possible to use a custom model as long as
-        is consistent with the required architecture, see
-        :class:`pytorch_widedeep.models.wide.Wide`
+        ``Wide`` model. This is a linear model where the non-linearities are
+        captured via crossed-columns.
     deeptabular: ``nn.Module``, Optional, default = None
-        currently ``pytorch-widedeep`` implements a number of possible
-        architectures for the ``deeptabular`` component. See the documenation
-        of the package. I recommend using the ``deeptabular`` components in
-        this package. However, it is possible to use a custom model as long
-        as is  consistent with the required architecture.
+        Currently this library implements a number of possible architectures
+        for the ``deeptabular`` component. See the documenation of the
+        package.
     deeptext: ``nn.Module``, Optional, default = None
-        Model for the text input. Must be an object of class ``DeepText``
-        or a custom model as long as is consistent with the required
-        architecture. See
-        :class:`pytorch_widedeep.models.deep_text.DeepText`
+        Currently this library implements a number of possible architectures
+        for the ``deeptext`` component. See the documenation of the
+        package.
     deepimage: ``nn.Module``, Optional, default = None
-        Model for the images input. Must be an object of class
-        ``DeepImage`` or a custom model as long as is consistent with the
-        required architecture. See
-        :class:`pytorch_widedeep.models.deep_image.DeepImage`
-    deephead: ``nn.Module``, Optional, default = None
-        Custom model by the user that will receive the outtput of the deep
-        component. Typically a FC-Head (MLP)
+        Currently this library uses ``torchvision`` and implements a number of
+        possible architectures for the ``deepimage`` component. See the
+        documenation of the package.
     head_hidden_dims: List, Optional, default = None
-        Alternatively, the ``head_hidden_dims`` param can be used to
-        specify the sizes of the stacked dense layers in the fc-head e.g:
-        ``[128, 64]``. Use ``deephead`` or ``head_hidden_dims``, but not
-        both.
-    head_dropout: float, default = 0.1
-        If ``head_hidden_dims`` is not None, dropout between the layers in
-        ``head_hidden_dims``
+        List with the sizes of the dense layers in the head e.g: [128, 64]
     head_activation: str, default = "relu"
-        If ``head_hidden_dims`` is not None, activation function of the head
-        layers. One of ``tanh``, ``relu``, ``gelu`` or ``leaky_relu``
+        Activation function for the dense layers in the head. Currently
+        `'tanh'`, `'relu'`, `'leaky_relu'` and `'gelu'` are supported
+    head_dropout: float, Optional, default = None
+        Dropout of the dense layers in the head
     head_batchnorm: bool, default = False
-        If ``head_hidden_dims`` is not None, specifies if batch
-        normalizatin should be included in the head layers
+        Boolean indicating whether or not to include batch normalization in
+        the dense layers that form the `'rnn_mlp'`
     head_batchnorm_last: bool, default = False
-        If ``head_hidden_dims`` is not None, boolean indicating whether or
-        not to apply batch normalization to the last of the dense layers
+        Boolean indicating whether or not to apply batch normalization to the
+        last of the dense layers in the head
     head_linear_first: bool, default = False
-        If ``head_hidden_dims`` is not None, boolean indicating whether
-        the order of the operations in the dense layer. If ``True``:
-        ``[LIN -> ACT -> BN -> DP]``. If ``False``: ``[BN -> DP -> LIN ->
-        ACT]``
+        Boolean indicating whether the order of the operations in the dense
+        layer. If ``True: [LIN -> ACT -> BN -> DP]``. If ``False: [BN -> DP ->
+        LIN -> ACT]``
+    deephead: ``nn.Module``, Optional, default = None
+        Alternatively, the user can pass a custom model that will receive the
+        output of the deep component. If ``deephead`` is not None all the
+        previous fc-head parameters will be ignored
     enforce_positive: bool, default = False
-        If final layer has activation function or not. Important if you are using
-        loss functions non-negative input restrictions, e.g. RMSLE, or if you know
-        your predictions are limited only to <0, inf)
+        Boolean indicating if the output from the final layer must be
+        positive. This is important if you are using loss functions with
+        non-negative input restrictions, e.g. RMSLE, or if you know your
+        predictions are bounded in between 0 and inf
     enforce_positive_activation: str, default = "softplus"
-        Activation function to enforce positive output from final layer. Use
-        "softplus" or "relu".
+        Activation function to enforce that the final layer has a positive
+        output. `'softplus'` or `'relu'` are supported.
     pred_dim: int, default = 1
         Size of the final wide and deep output layer containing the
         predictions. `1` for regression and binary classification or number
         of classes for multiclass classification.
+    with_fds: bool, default = False
+        If feature distribution smoothing (FDS) should be applied before the
+        final prediction layer. Only available for regression problems. See
+        `Delving into Deep Imbalanced Regression <https://arxiv.org/abs/2102.09554>`_
+        for details.
+    fds_config: dict, default = None
+        Dictionary defining specific values for
+        ``FeatureDistributionSmoothing`` layer
 
     Examples
     --------
 
-    >>> from pytorch_widedeep.models import TabResnet, DeepImage, DeepText, Wide, WideDeep
+    >>> from pytorch_widedeep.models import TabResnet, Vision, BasicRNN, Wide, WideDeep
     >>> embed_input = [(u, i, j) for u, i, j in zip(["a", "b", "c"][:4], [4] * 3, [8] * 3)]
     >>> column_idx = {k: v for v, k in enumerate(["a", "b", "c"])}
     >>> wide = Wide(10, 1)
-    >>> deeptabular = TabResnet(blocks_dims=[8, 4], column_idx=column_idx, embed_input=embed_input)
-    >>> deeptext = DeepText(vocab_size=10, embed_dim=4, padding_idx=0)
-    >>> deepimage = DeepImage(pretrained=False)
+    >>> deeptabular = TabResnet(blocks_dims=[8, 4], column_idx=column_idx, cat_embed_input=embed_input)
+    >>> deeptext = BasicRNN(vocab_size=10, embed_dim=4, padding_idx=0)
+    >>> deepimage = Vision()
     >>> model = WideDeep(wide=wide, deeptabular=deeptabular, deeptext=deeptext, deepimage=deepimage)
 
 
-    .. note:: While I recommend using the ``wide`` and ``deeptabular`` components
-        within this package when building the corresponding model components,
-        it is very likely that the user will want to use custom text and image
-        models. That is perfectly possible. Simply, build them and pass them
-        as the corresponding parameters. Note that the custom models MUST
-        return a last layer of activations (i.e. not the final prediction) so
-        that  these activations are collected by ``WideDeep`` and combined
-        accordingly. In addition, the models MUST also contain an attribute
-        ``output_dim`` with the size of these last layers of activations. See
-        for example :class:`pytorch_widedeep.models.tab_mlp.TabMlp`
-
+    .. note:: It is possible to use custom components to build Wide & Deep models.
+        Simply, build them and pass them as the corresponding parameters. Note
+        that the custom models MUST return a last layer of activations
+        (i.e. not the final prediction) so that  these activations are
+        collected by ``WideDeep`` and combined accordingly. In addition, the
+        models MUST also contain an attribute ``output_dim`` with the size of
+        these last layers of activations. See for
+        example :class:`pytorch_widedeep.models.tab_mlp.TabMlp`
     """
 
     def __init__(
@@ -141,10 +138,12 @@ class WideDeep(nn.Module):
         enforce_positive: bool = False,
         enforce_positive_activation: str = "softplus",
         pred_dim: int = 1,
+        with_fds: bool = False,
+        **fds_config,
     ):
         super(WideDeep, self).__init__()
 
-        self._check_model_components(
+        self._check_inputs(
             wide,
             deeptabular,
             deeptext,
@@ -152,6 +151,7 @@ class WideDeep(nn.Module):
             deephead,
             head_hidden_dims,
             pred_dim,
+            with_fds,
         )
 
         # required as attribute just in case we pass a deephead
@@ -164,34 +164,49 @@ class WideDeep(nn.Module):
         self.deepimage = deepimage
         self.deephead = deephead
         self.enforce_positive = enforce_positive
+        self.with_fds = with_fds
 
         if self.deeptabular is not None:
             self.is_tabnet = deeptabular.__class__.__name__ == "TabNet"
         else:
             self.is_tabnet = False
 
-        if self.deephead is None:
-            if head_hidden_dims is not None:
-                self._build_deephead(
-                    head_hidden_dims,
-                    head_activation,
-                    head_dropout,
-                    head_batchnorm,
-                    head_batchnorm_last,
-                    head_linear_first,
-                )
-            else:
-                self._add_pred_layer()
+        if self.deephead is None and head_hidden_dims is not None:
+            self._build_deephead(
+                head_hidden_dims,
+                head_activation,
+                head_dropout,
+                head_batchnorm,
+                head_batchnorm_last,
+                head_linear_first,
+            )
+        elif self.deephead is not None:
+            pass
+        else:
+            self._add_pred_layer()
+
+        if self.with_fds:
+            self.fds_layer = FDSLayer(feature_dim=self.deeptabular.output_dim, **fds_config)  # type: ignore[arg-type]
 
         if self.enforce_positive:
             self.enf_pos = get_activation_fn(enforce_positive_activation)
 
-    def forward(self, X: Dict[str, Tensor]):
+    def forward(
+        self,
+        X: Dict[str, Tensor],
+        y: Optional[Tensor] = None,
+        epoch: Optional[int] = None,
+    ):
+
+        if self.with_fds:
+            return self._forward_deep_with_fds(X, y, epoch)
+
         wide_out = self._forward_wide(X)
         if self.deephead:
             deep = self._forward_deephead(X, wide_out)
         else:
             deep = self._forward_deep(X, wide_out)
+
         if self.enforce_positive:
             return self.enf_pos(deep)
         else:
@@ -223,12 +238,14 @@ class WideDeep(nn.Module):
             head_batchnorm_last,
             head_linear_first,
         )
+
         self.deephead.add_module(
             "head_out", nn.Linear(head_hidden_dims[-1], self.pred_dim)
         )
 
     def _add_pred_layer(self):
-        if self.deeptabular is not None:
+        # if FDS the FDS Layer already includes the pred layer
+        if self.deeptabular is not None and not self.with_fds:
             if self.is_tabnet:
                 self.deeptabular = nn.Sequential(
                     self.deeptabular,
@@ -300,8 +317,24 @@ class WideDeep(nn.Module):
 
         return res
 
+    def _forward_deep_with_fds(
+        self,
+        X: Dict[str, Tensor],
+        y: Optional[Tensor] = None,
+        epoch: Optional[int] = None,
+    ):
+        res = self.fds_layer(self.deeptabular(X["deeptabular"]), y, epoch)
+        if self.enforce_positive:
+            if isinstance(res, Tuple):  # type: ignore[arg-type]
+                out = (res[0], self.enf_pos(res[1]))
+            else:
+                out = self.enf_pos(res)
+        else:
+            out = res
+        return out
+
     @staticmethod  # noqa: C901
-    def _check_model_components(  # noqa: C901
+    def _check_inputs(  # noqa: C901
         wide,
         deeptabular,
         deeptext,
@@ -309,6 +342,7 @@ class WideDeep(nn.Module):
         deephead,
         head_hidden_dims,
         pred_dim,
+        with_fds,
     ):
 
         if wide is not None:
@@ -378,3 +412,17 @@ class WideDeep(nn.Module):
                     deephead_inp_feat, output_dim
                 )
             )
+
+        if with_fds and (
+            (
+                wide is not None
+                or deeptext is not None
+                or deepimage is not None
+                or deephead is not None
+            )
+            or pred_dim != 1
+        ):
+            raise ValueError(
+                """Feature Distribution Smoothing (FDS) is supported when using only a deeptabular component"
+                " and for regression problems."""
+            )
