@@ -5,8 +5,8 @@ import torch
 import pytest
 
 from pytorch_widedeep.wdtypes import WideDeep
-from pytorch_widedeep.models.tabnet._utils import create_explain_matrix
-from pytorch_widedeep.models.tabnet.tab_net import TabNet  # noqa: F403
+from pytorch_widedeep.models.tabular.tabnet._utils import create_explain_matrix
+from pytorch_widedeep.models.tabular.tabnet.tab_net import TabNet  # noqa: F403
 
 # I am going over test this model due to the number of components
 
@@ -17,7 +17,9 @@ n_cols = 2
 batch_size = 10
 colnames = list(string.ascii_lowercase)[: (n_cols * 2)]
 embed_cols = [np.random.choice(np.arange(n_embed), batch_size) for _ in range(n_cols)]
+embed_input = [(u, i, 1) for u, i in zip(colnames[:2], [n_embed] * 2)]
 cont_cols = [np.random.rand(batch_size) for _ in range(n_cols)]
+continuous_cols = colnames[-5:]
 
 X_tab = torch.from_numpy(np.vstack(embed_cols + cont_cols).transpose())
 X_tab_emb = X_tab[:, :n_cols]
@@ -27,18 +29,15 @@ X_tab_cont = X_tab[:, n_cols:]
 # Test functioning using the defaults
 ###############################################################################
 
-embed_input = [(u, i, 1) for u, i in zip(colnames[:2], [n_embed] * 2)]
-
-model1 = TabNet(
-    column_idx={k: v for v, k in enumerate(colnames)},
-    embed_input=embed_input,
-    continuous_cols=colnames[n_cols:],
-)
-
 
 def test_embeddings_have_padding():
+    model = TabNet(
+        column_idx={k: v for v, k in enumerate(colnames)},
+        cat_embed_input=embed_input,
+        continuous_cols=colnames[n_cols:],
+    )
     res = []
-    for k, v in model1.cat_embed_and_cont.embed_layers.items():
+    for k, v in model.cat_and_cont_embed.cat_embed.embed_layers.items():
         res.append(v.weight.size(0) == n_embed + 1)
         res.append(not torch.all(v.weight[0].bool()))
     assert all(res)
@@ -55,12 +54,30 @@ def test_embeddings_have_padding():
 def test_tabnet_output(cont_norm_layer):
     model = TabNet(
         column_idx={k: v for v, k in enumerate(colnames)},
-        embed_input=embed_input,
+        cat_embed_input=embed_input,
         continuous_cols=colnames[n_cols:],
         cont_norm_layer=cont_norm_layer,
     )
     out1, out2 = model(X_tab)
-    assert out1.size(0) == 10 and out1.size(1) == model1.step_dim
+    assert out1.size(0) == 10 and out1.size(1) == model.step_dim
+
+
+@pytest.mark.parametrize(
+    "embed_continuous",
+    [
+        True,
+        False,
+    ],
+)
+def test_tabnet_embed_continuos(embed_continuous):
+    model = TabNet(
+        column_idx={k: v for v, k in enumerate(colnames)},
+        cat_embed_input=embed_input,
+        continuous_cols=colnames[n_cols:],
+        embed_continuous=embed_continuous,
+    )
+    out1, out2 = model(X_tab)
+    assert out1.size(0) == 10 and out1.size(1) == model.step_dim
 
 
 ###############################################################################
@@ -78,12 +95,12 @@ def test_tabnet_output(cont_norm_layer):
 def test_mask_type(mask_type):
     model = TabNet(
         column_idx={k: v for v, k in enumerate(colnames)},
-        embed_input=embed_input,
+        cat_embed_input=embed_input,
         continuous_cols=colnames[n_cols:],
         mask_type=mask_type,
     )
     out1, out2 = model(X_tab)
-    assert out1.size(0) == 10 and out1.size(1) == model1.step_dim
+    assert out1.size(0) == 10 and out1.size(1) == model.step_dim
 
 
 ###############################################################################
@@ -101,12 +118,12 @@ def test_mask_type(mask_type):
 def test_ghost_bn(ghost_bn):
     model = TabNet(
         column_idx={k: v for v, k in enumerate(colnames)},
-        embed_input=embed_input,
+        cat_embed_input=embed_input,
         continuous_cols=colnames[n_cols:],
         ghost_bn=ghost_bn,
     )
     out1, out2 = model(X_tab)
-    assert out1.size(0) == 10 and out1.size(1) == model1.step_dim
+    assert out1.size(0) == 10 and out1.size(1) == model.step_dim
 
 
 ###############################################################################
@@ -115,12 +132,17 @@ def test_ghost_bn(ghost_bn):
 
 
 def test_forward_masks():
-    out1, out2 = model1.forward_masks(X_tab)
+    model = TabNet(
+        column_idx={k: v for v, k in enumerate(colnames)},
+        cat_embed_input=embed_input,
+        continuous_cols=colnames[n_cols:],
+    )
+    out1, out2 = model.forward_masks(X_tab)
     bsz, nfeat = X_tab.shape[0], X_tab.shape[1]
     out = []
     out.append(out1.shape[0] == bsz)
     out.append(out1.shape[1] == nfeat)
-    for step in range(model1.n_steps):
+    for step in range(model.n_steps):
         out.append(out2[step].size(0) == bsz)
         out.append(out2[step].size(1) == nfeat)
     assert all(out)
@@ -131,28 +153,59 @@ def test_forward_masks():
 ###############################################################################
 
 
-def test_create_explain_matrix():
-    embed_input = [(u, i, 2) for u, i in zip(colnames[:2], [n_embed] * 2)]
-    continuous_cols = colnames[2:]
-    embed_cols = colnames[:2]
-    column_idx = {k: v for v, k in enumerate(colnames)}
+@pytest.mark.parametrize(
+    "w_cat, w_cont, embed_continuous",
+    [
+        (True, True, False),
+        (True, True, True),
+        (False, True, False),
+        (False, True, True),
+        (True, False, False),
+        (True, False, False),
+    ],
+)
+def test_create_explain_matrix(w_cat, w_cont, embed_continuous):
+
+    if w_cat and w_cont:
+        cat_embed_input = [(u, i, 4) for u, i in zip(colnames[:2], [n_embed] * 2)]
+        continuous_cols = colnames[2:]
+        if embed_continuous:
+            embed_cols = colnames
+        else:
+            embed_cols = colnames[:2]
+        column_idx = {k: v for v, k in enumerate(colnames)}
+    elif w_cat:
+        cat_embed_input = [(u, i, 4) for u, i in zip(colnames[:2], [n_embed] * 2)]
+        continuous_cols = None
+        embed_cols = colnames[:2]
+        column_idx = {k: v for v, k in enumerate(colnames[:2])}
+    elif w_cont:
+        cat_embed_input = None
+        continuous_cols = colnames[2:]
+        column_idx = {k: v for v, k in enumerate(colnames[2:])}
+        if embed_continuous:
+            embed_cols = colnames[2:]
+        else:
+            embed_cols = []
 
     tabnet = TabNet(
         column_idx=column_idx,
-        embed_input=embed_input,
+        cat_embed_input=cat_embed_input,
         continuous_cols=continuous_cols,
+        embed_continuous=embed_continuous,
+        cont_embed_dim=4,
     )
     wdmodel = WideDeep(deeptabular=tabnet)
 
     expl_mtx = create_explain_matrix(wdmodel)
 
     checks = []
-    checks.append(expl_mtx.sum() == tabnet.embed_and_cont_dim)
+    checks.append(expl_mtx.sum() == tabnet.embed_out_dim)
     checks.append(all(expl_mtx.sum(1) == 1))
     for col, idx in column_idx.items():
         if col in embed_cols:
-            checks.append(expl_mtx[:, idx].sum() == 2.0)
-        elif col in continuous_cols:
+            checks.append(expl_mtx[:, idx].sum() == 4.0)
+        elif not embed_continuous and col in continuous_cols:
             checks.append(expl_mtx[:, idx].sum() == 1.0)
 
     assert all(checks)
